#ifndef _NIVEL_
#define _NIVEL_

#include <Gosu/Gosu.hpp>
#include <boost/lexical_cast.hpp>
#include <iostream>
#include <fstream>
#include "sonido.h"
#include "completado.h"

/**
 * @brief Clase Nivel
 * 
 *@author Lourdes
*/

class Nivel{
private:  
  Gosu::Sample * sonido_cerdo;
  
  vector<Gosu::Image*> granjero_der; //granjero
  vector<Gosu::Image*> granjero_frente;
  vector<Gosu::Image*> granjero_izq;
  vector<Gosu::Image*> granjero_espalda;
  
  Gosu::Image * cerdo; //cerdo
  Gosu::Image * bellota; //bellota
  Gosu::Image * esquina_tra_izq; //esquina_tra_izq
  Gosu::Image * esquina_tra_der; //esquina_tra_der
  Gosu::Image * esquina_del_izq; //esquina_del_izq
  Gosu::Image * esquina_del_der; //esquina_del_der		
  Gosu::Image * valla_izq; //valla_izq		
  Gosu::Image * valla_tras; //valla_tra
  Gosu::Image * valla_del;  //valla_del	
  Gosu::Image * valla_der; //valla_der
  Gosu::Image * valla_menu; //valla_menu
  
  Gosu::Font * texto;
  Sonido& sonnivel_;

  
public:
   /**Constructor de Nivel
    *@param Gosu::Graphics, Sonido::sonido  
    */ 
  Nivel(Gosu::Graphics & graphics,Sonido & sonido):sonnivel_(sonido){
    
    sonido_cerdo=new Gosu::Sample(L"../sonido/cerdo1.ogg");
    cerdo = new Gosu::Image(graphics, L"../Imagenes/cerdito.png");
    valla_tras = new Gosu::Image(graphics, L"../Imagenes/valla_trasera.png");
    valla_del = new Gosu::Image(graphics, L"../Imagenes/valla_delantera.png");
    valla_izq = new Gosu::Image(graphics, L"../Imagenes/valla_izquierda.png");
    esquina_tra_izq = new Gosu::Image(graphics,L"../Imagenes/esquina_tra_izq.png");
    esquina_tra_der = new Gosu::Image(graphics,L"../Imagenes/esquina_tra_der.png");
    esquina_del_izq = new Gosu::Image(graphics,L"../Imagenes/esquina_del_izq.png");
    esquina_del_der = new Gosu::Image(graphics,L"../Imagenes/esquina_del_der.png");
    bellota = new Gosu::Image(graphics,L"../Imagenes/bellota.png");
    valla_der =new Gosu::Image(graphics,L"../Imagenes/valla_derecha.png");
    valla_menu= new Gosu::Image(graphics,L"../Imagenes/valla_menu.png");

    texto= new Gosu::Font(graphics,L"../Imagenes/letra.ttf",30,0);

    granjero_der.push_back(new Gosu::Image(graphics, L"../Imagenes/definitivos/granjeros/derecha1.png"));
    granjero_der.push_back(new Gosu::Image(graphics, L"../Imagenes/definitivos/granjeros/derecha2.png"));
    granjero_der.push_back(new Gosu::Image(graphics, L"../Imagenes/definitivos/granjeros/derecha3.png"));	

    granjero_frente.push_back(new Gosu::Image(graphics, L"../Imagenes/definitivos/granjeros/frente1.png"));
    granjero_frente.push_back(new Gosu::Image(graphics, L"../Imagenes/definitivos/granjeros/frente2.png"));
    granjero_frente.push_back(new Gosu::Image(graphics, L"../Imagenes/definitivos/granjeros/frente3.png"));
    
    granjero_izq.push_back(new Gosu::Image(graphics, L"../Imagenes/definitivos/granjeros/izquierda1.png"));
    granjero_izq.push_back(new Gosu::Image(graphics, L"../Imagenes/definitivos/granjeros/izquierda2.png"));
    granjero_izq.push_back(new Gosu::Image(graphics, L"../Imagenes/definitivos/granjeros/izquierda3.png"));	

    granjero_espalda.push_back(new Gosu::Image(graphics, L"../Imagenes/definitivos/granjeros/espalda1.png"));
    granjero_espalda.push_back(new Gosu::Image(graphics, L"../Imagenes/definitivos/granjeros/espalda2.png"));
    granjero_espalda.push_back(new Gosu::Image(graphics, L"../Imagenes/definitivos/granjeros/espalda3.png"));
  }
   
   /**Destructor de Nivel*/
  ~Nivel(){
    delete sonido_cerdo;
  }
 
  /**
   * Funcion que se encarga dibujar el escenario cuando estamos en un nivel.
   * @param Tablero::tablero*, int &, int &, float & 
   */
  void draw(tablero *t, int & mira_, int & animacion_, float puntuacion_total_){
    for(int i = 0; i < 12; i++){ 
      for(int j = 0; j < 12; j++){   //Recoremos el tablero y dependiendo del numero dibujamos la imagen correspondiente.
	if((t->bellotas_[i][j])==1)  //miramos si hay una bellota
	  bellota->draw(32*(j+1),32*i+150,1,true); //la dibujamos donde corresponde.

	if((t->tablero_[i][j])==1){	 //si es el granjero
	  if(mira_ == 0){
	    granjero_frente[0]->draw(32*(j+1),32*(i-1)+150,j+1,true);
	  }
	  if(mira_ == 1){
	    if(animacion_ > 0){
		    granjero_frente[Gosu::milliseconds() / 100 % 3]->draw(32*(j+1),(32*(i-1)-animacion_)+150,j+1,true);
		    animacion_--;				
	    }
	    else
		    granjero_frente[0]->draw(32*(j+1),32*(i-1)+150,j+1,true);
		    
	  }
	  if(mira_ == 2){
	    if(animacion_ > 0){
		    granjero_der[Gosu::milliseconds() / 100 % 3]->draw((32*(j+1)-animacion_),32*(i-1)+150,j+1,true);
		    animacion_--;				
	    }
	    else
		    granjero_der[0]->draw(32*(j+1),32*(i-1)+150,j+1,true);
	  }
	  if(mira_ == 3){
	    if(animacion_ > 0){
	      granjero_izq[Gosu::milliseconds() / 100 % 3]->draw((32*(j+1)+animacion_),32*(i-1)+150,j+1,true);
	      animacion_--;				
	    }
	    else
	      granjero_izq[0]->draw(32*(j+1),32*(i-1)+150,j+1,true);
	  }
	  if(mira_ == 4){
	    if(animacion_ > 0){
	      granjero_espalda[Gosu::milliseconds() / 100 % 3]->draw(32*(j+1),(32*(i-1)+animacion_)+150,j+1,true);
	      animacion_--;				
	    }
	    else
	      granjero_espalda[0]->draw(32*(j+1),32*(i-1)+150,j+1,true);
	  }
	}
	if((t->tablero_[i][j])==2){	//si es un cerdo
	  cerdo->draw(32*(j+1),32*i+150,j,true);
	}
	if((t->tablero_[i][j])==4) //si es la valla trasera
	  valla_tras->draw(32*(j+1),32*(i)+150,j,true);   	
	if((t->tablero_[i][j])==5) //si es la valla delantera
	  valla_del->draw(32*(j+1),32*(i)+150,j,true); 
	if((t->tablero_[i][j])==6)  //si es la valla derecha
	  valla_der->draw(32*(j+1),32*(i)+150,j,true);   
	if((t->tablero_[i][j])==7) //si es la valla izquierda
	  valla_izq->draw(32*(j+1),32*(i)+150,j,true);     	
	if((t->tablero_[i][j])==8) //si es la esquina trasera izquierda
	  esquina_tra_izq->draw(32*(j+1),32*i+150,j,true);  
	if((t->tablero_[i][j])==9) //si es la esquina trasera derecha
	  esquina_tra_der->draw(32*(j+1),32*i+150,j,true);
	if((t->tablero_[i][j])==10) //si es la esquina delantera izquierda
	  esquina_del_izq->draw(32*(j+1),32*i+150,j,true);  
	if((t->tablero_[i][j])==11) //si es la esquina delantera derecha
	  esquina_del_der->draw(32*(j+1),32*i+150,j,true);				
	
      }  
    }

    valla_menu-> draw(550,275, 1);
    texto->draw(L"Menu",630, 355, 1);
    texto->draw(L"Reintentar",600, 386, 1);
    texto->draw(L"Pausar musica",590, 410, 1);
    texto->draw(L"Pasos Minimos: "+boost::lexical_cast<wstring>(t->pasos_),550,245,1);
    texto->draw(L"Pasos: "+boost::lexical_cast<wstring>(t->pasos_andados),600,265,1);
    float aux = (puntuacion_total_/12);
    texto->draw(L"Puntuacion: "+boost::lexical_cast<wstring>(aux),550,285,1);
  }
  
  /**
   * Funcion que se encarga del manejo de botones cuando estamos en el menu principal.
   * @param Gosu::Button & B, Gosu::Input & input, int &, int &, int &, int &, string, tablero*, int &, int &, int &, float & 
   */
  void buttonDown(Gosu::Button & B, Gosu::Input & input, int & animacion_, int & mira_, int & menu_, int & principal_, string & nivel_, tablero* t, int & guardado_,int & completo_,int & puntuacion, float & puntuacion_total_){
    int nivel_actual = t->nivel;
    if(B == Gosu::kbDown){
      animacion_ = 30;
      mira_ = 1;
      bool cerdo = t->mover(1,0);
      if(cerdo && !sonnivel_.silencio())
	sonido_cerdo->play(true);
	
      if(t->comprobar()==true){ //si se ha ganado el nivel, volvemos al menu de niveles
	//menu_=1;
	completo_=1;
	
	nivel_="0";
	puntuacion=t->puntuacion_;
	puntuacion_total_ = puntuacion_total_ + puntuacion;
	sonnivel_.cambiarfondo();
	if( nivel_actual == guardado_){
	guardado_++;
	}
      }
    }
    else if(B == Gosu::kbRight){
      animacion_ = 30;
      mira_ = 2;
      bool cerdo = t->mover(0,1);
      if(cerdo && !sonnivel_.silencio())
				sonido_cerdo->play(true);
      if(t->comprobar()==true){ //si se ha ganado el nivel, volvemos al menu de niveles
	completo_=1;				
	//menu_=1;
	
	nivel_="0"; 
	puntuacion=t->puntuacion_;
	puntuacion_total_ = puntuacion_total_ + puntuacion;
	sonnivel_.cambiarfondo();

	if( nivel_actual == guardado_){
	  guardado_++;
	}
      }
    }
    else if(B == Gosu::kbLeft){
      animacion_ = 30;
      mira_ = 3;
      bool cerdo = t->mover(0,-1);
      if(cerdo && !sonnivel_.silencio())
				sonido_cerdo->play(true);
      if(t->comprobar()==true){ //si se ha ganado el nivel, volvemos al menu de niveles
	completo_=1;				
	//menu_=1;
	
	nivel_="0";
	puntuacion=t->puntuacion_;
	puntuacion_total_ = puntuacion_total_ + puntuacion;
	sonnivel_.cambiarfondo();

	if( nivel_actual == guardado_){
	  guardado_++;	
	}
      }
    }
    else if(B == Gosu::kbUp){
      animacion_ = 30;
      mira_ = 4;
      bool cerdo = t->mover(-1,0);
      if(cerdo && !sonnivel_.silencio())
				sonido_cerdo->play(true);
      if(t->comprobar()==true){ //si se ha ganado el nivel, volvemos al menu de niveles
	completo_=1;
	//menu_=1;
	nivel_="0";
	puntuacion=t->puntuacion_;
	puntuacion_total_ = puntuacion_total_ + puntuacion;
	sonnivel_.cambiarfondo();

	if( nivel_actual == guardado_){
	  guardado_++;
	}
      }
      
    }
    if(B== Gosu::msLeft){
      //tomamos las coordenadas del cursor
      int mX=input.mouseX();        
      int mY=input.mouseY();        
      if((mX > 576 && mX < 748) && (mY > 358 && mY < 383))
      {
	principal_ =0;  //ponemos el menu principal a 1 porque es la primera ventana que se muestra
	nivel_ ="0"; //ponemos el menu a o porque inicialemente no queremos tener ningun nivel visible
	menu_ = 1;
	sonnivel_.cambiarfondo();
      }
      if((mX > 583 && mX < 762) && (mY > 388 && mY < 412))
      {
	t->restaurar();
      }
      if((mX > 549 && mX < 742) && (mY > 417 && mY < 444))
      {
	sonnivel_.silenciar();
      } 
    }
  }
};
#endif
